import {
  select,
  forceSimulation,
  forceManyBody,
  forceLink,
  forceCenter,
  drag,
  zoom,
} from 'd3';

import { nodes, links, MANY_BODY_STRENGTH } from './data';

const svg = select('#container')

const width = +svg.attr('width');
const height = +svg.attr('height');
const centerX = width / 2;
const centerY = height / 2;

const simulation = forceSimulation(nodes)
  .force('charge', forceManyBody().strength(MANY_BODY_STRENGTH))
  .force(
    'link',
    forceLink(links).distance((link) => link.distance)
  )
  .force('center', forceCenter(centerX, centerY));

const dragInteraction = drag().on('drag', (event, node) => {
  node.fx = event.x;
  node.fy = event.y;
  simulation.alpha(1);
  simulation.restart();
});

let g = svg.append("g")
    // .attr("class", "everything");

const lines = g.append('g')
  // .attr("class", "links")
  .selectAll('line')
  .data(links)
  .enter()
  .append('line')
  .attr('stroke', (link) => link.color || 'black');


const circles = g.append('g')
  // .attr("class", "nodes") 
  .selectAll('circle')
  .data(nodes)
  .enter()
  .append('circle')
  .attr('fill', (node) => node.color || 'gray')
  .attr('r', (node) => node.size)
  .style('cursor', 'pointer')
  // .call(dragInteraction)
	.on('click', function(d,i) {
    const id = i.id
    window.alert(`https://vk.com/id` + id)
    	// window.location.href = `https://vk.com/id` + id
            // handle events here
            // d - datum
            // i - identifier or index
            // this - the `<rect>` that was clicked
    });

const text = g.append('g')
  .selectAll('text')
  .data(nodes)
  .enter()
  .append('text')
  .attr('text-anchor', 'middle')
  .attr('alignment-baseline', 'middle')
  .style('pointer-events', 'none')
  .style('font-weight', 500)
  .text((node) => node.name);

let drag_handler = drag()
	.on("start", drag_start)
	.on("drag", drag_drag)
	.on("end", drag_end);	

drag_handler(circles)

let zoom_handler = zoom()
    .on("zoom", zoom_actions);

zoom_handler(svg);

function drag_start(event, d) {
 if (!event.active) simulation.alpha(0.3).restart();
    d.fx = d.x;
    d.fy = d.y;
}

//make sure you can't drag the circle outside the box
function drag_drag(event, d) {
  d.fx = event.x;
  d.fy = event.y;
}

function drag_end(event, d) {
  if (!event.active) simulation.alphaTarget(0);
  d.fx = null;
  d.fy = null;
}

//Zoom functions 
function zoom_actions(e){
    g.attr("transform", e.transform)
}

simulation.on('tick', (e) => {
  circles.attr('cx', (node) => node.x).attr('cy', (node) => node.y);
  text.attr('x', (node) => node.x).attr('y', (node) => node.y - 30);

  lines
    .attr('x1', (link) => link.source.x)
    .attr('y1', (link) => link.source.y)
    .attr('x2', (link) => link.target.x)
    .attr('y2', (link) => link.target.y);
});

